/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef GB2_EXTERNAL_TOOL_REGISTRY_H
#define GB2_EXTERNAL_TOOL_REGISTRY_H

#include <QtCore/QString>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QVariant>

#include <core_api/IdRegistry.h>
#include <core_api/core_api.h>
#include <core_api/ObjectViewModel.h>

namespace GB2 {


class GB2_COREAPI_EXPORT ExternalTool : public QObject {
    Q_OBJECT
public:
    ExternalTool(QString name, QString path = "");
    ~ExternalTool();

    const QString& getName()  const { return name; }
    const QString& getPath()  const { return path; }
    const QIcon&   getIcon()  const { return icon; }
    const QIcon&   getGrayIcon()  const { return grayIcon; }
    const QString& getDescription()  const { return description; }

    void setPath(const QString& _path);

signals:
    void si_pathChanged();

protected:
    QString     name;
    QString     path;
    QIcon       icon;
    QIcon       grayIcon;
    QString     description;

}; // ExternalTool


class GB2_COREAPI_EXPORT ExternalToolRegistry : public QObject {
    Q_OBJECT
public:
    ~ExternalToolRegistry();

    ExternalTool* getByName(const QString& id);

    bool registerEntry(ExternalTool* t);
    void unregisterEntry(const QString& id);

    QList<ExternalTool*> getAllEntries() const;

    void setTemporaryDir(QString _temporyDirectory);
    const QString& getTemporaryDir() const { return temporaryDirectory; }

signals:
    void si_temporyDirChanged();

protected:
    QMap<QString, ExternalTool*>    registry;
    QString                         temporaryDirectory;

}; // ExternalToolRegistry

} //namespace
#endif // GB2_EXTERNAL_TOOL_REGISTRY_H
