/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _CLOUD_REMOTE_MACHINE_H_
#define _CLOUD_REMOTE_MACHINE_H_

#include <QtCore/QEventLoop>
#include <QtCore/QPointer>
#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkReply>
#include <QtNetwork/QNetworkProxy>

#include <memory>

#include <distributed_computing/RemoteMachine.h>

namespace GB2 {

class CloudRemoteMachineSettings : public RemoteMachineSettings
{
public:
    CloudRemoteMachineSettings();
    CloudRemoteMachineSettings(const QString &url, const QString& username, const QString& passwd);

    virtual bool isPingTaskSupported() { return false; }
    
    virtual QString toString() const;
    virtual bool operator==( const RemoteMachineSettings& machine ) const;

    QString getUrl()const {return url; }
    QString getUserName() const { return name;}
    QString getPasswd() const {return passwd; }
    QString getInfo() const { return url; }
    QString getSessionId() const { return sessionId; }
    void setSessionId(const QString& id) { sessionId = id; }

    QString serialize()const;
    bool deserialize( const QString & data );

private:
    QString url;
    QString name;
    QString passwd;
    QString sessionId;
};

class Uctp;
class UctpSession;

class CloudRemoteMachine : public QObject, public RemoteMachine
{
    Q_OBJECT
public:
    CloudRemoteMachine(CloudRemoteMachineSettings* s);
    virtual ~CloudRemoteMachine();
    virtual RemoteTaskError runRemoteTask( const QString & taskFactoryId, const QVariant & settings, qint64 * taskId );
    virtual RemoteTaskError cancelRemoteTask( qint64 taskId );
    virtual RemoteTaskError deleteRemoteTask( qint64 taskId );
    virtual RemoteTaskError getRemoteTaskCancelFlag( qint64 taskId, bool * cancelFlag );
    virtual RemoteTaskError getRemoteTaskState( qint64 taskId, Task::State * state );
    virtual RemoteTaskError getRemoteTaskProgress( qint64 taskId, int * progress );
    virtual RemoteTaskError getRemoteTaskResult( qint64 taskId, QVariant * resultUrls );
    virtual RemoteTaskError getRemoteTaskError( qint64 taskId, QString * errMsg );
    virtual RemoteTaskError getUuid( QUuid * uuid );
    virtual RemoteTaskError getServicesList( QStringList * list );
    virtual RemoteTaskError getHostName( QString * hostname );

private slots:
    void sl_onReply();
    
private:
    // Data
    CloudRemoteMachineSettings*      settings;
    QNetworkProxy                   proxy;
    
    QPointer<QEventLoop>            eventLoop;
    QPointer<QNetworkReply>         reply;
    QByteArray                      activeCommand;
    
    std::auto_ptr<Uctp>             protocolHandler;
    std::auto_ptr<UctpSession>      session;
    bool                            errorOccured;
    QString                         errorMessage;
    QVariantMap                     replyData;
    QString                            cloudUrl;
    // Methods
    RemoteTaskError initSession();
    void sendRequest(const QByteArray& command, const QByteArray& data);
    void saveResult(const QByteArray& result, const QString& path);
    void updateGlobalSettings();

};


class CloudRemoteMachineFactory : public RemoteMachineFactory
{
public:
    CloudRemoteMachineFactory();
    virtual ~CloudRemoteMachineFactory();

    virtual RemoteMachine * createInstance( const QString & serializedSettings ) const;
    virtual RemoteMachine * createInstance( RemoteMachineSettings * settings ) const;
    virtual RemoteMachineSettings * createSettings( const QString & serializedSettings ) const;
    
};



} // namespace GB2

#endif // _CLOUD_REMOTE_MACHINE_H_
