/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include <QtCore/QFile>

#include <core_api/GUrl.h>

#include "UctpRequestBuilders.h"

namespace GB2 {



InitSessionRequest::InitSessionRequest(const UctpCommand& cmd, const QString& username, const QString& passwd )
: UctpRequestBuilder(cmd), name(username), pass(passwd)
{

}

void InitSessionRequest::formHeader( QXmlStreamWriter& stream )
{

}

void InitSessionRequest::formContents( QXmlStreamWriter& stream )
{
    stream.writeTextElement(UctpElements::USER_NAME, name);
    stream.writeTextElement(UctpElements::PASSWD, pass);
}

//////////////////////////////////////////////////////////////////////////



RunRemoteTaskRequest::RunRemoteTaskRequest( const UctpCommand& cmd, UctpSession* s, const QByteArray& ts, const QStringList& input, const QStringList& res )
:UctpRequestBuilder(cmd), session(s), taskSchema(ts), inputUrls(input), resultUrls(res)
{
    foreach (const QString& url, inputUrls) {
        QFile file(url);
        if (!file.open(QIODevice::ReadOnly)) {
            throw UctpError(QString("Can not open data from file %1").arg(url));
        }
        QByteArray rawData = file.readAll();
        if (rawData.size() == 0) {
            throw UctpError(QString("Can not read data from file %1").arg(url));
        }
        dataMap.insert( url, rawData.toBase64() );
    }

}

void RunRemoteTaskRequest::formHeader( QXmlStreamWriter& stream )
{
    stream.writeTextElement(UctpElements::SESSION_ID, session->getUid());
}

void RunRemoteTaskRequest::formContents( QXmlStreamWriter& stream )
{
     stream.writeStartElement(UctpElements::TASK_SCHEMA);
     stream.writeAttribute(UctpAttributes::ENCODING, UctpValues::BASE64);
     stream.writeCharacters(taskSchema.toBase64());
     stream.writeEndElement(); 
     
     foreach(const QString& url, resultUrls) {
         stream.writeStartElement(UctpElements::RESULT);
         stream.writeAttribute(UctpAttributes::NAME, GUrl(url).fileName());
         stream.writeEndElement();

     }

     foreach(const QString url, dataMap.keys()) {
         const QByteArray& data = dataMap.value(url);
         stream.writeStartElement(UctpElements::DATA);
         stream.writeAttribute(UctpAttributes::IS_GZIPPED, UctpValues::UCTP_FALSE);
         stream.writeAttribute(UctpAttributes::ENCODING, UctpValues::BASE64);
         stream.writeAttribute(UctpAttributes::NAME, GUrl(url).fileName());
         stream.writeAttribute(UctpAttributes::SIZE, QString("%1").arg(data.size()));
         stream.writeCharacters(dataMap.value(url));
         stream.writeEndElement();
     }

}

//////////////////////////////////////////////////////////////////////////

void EndSessionRequest::formHeader( QXmlStreamWriter& stream )
{
    stream.writeTextElement(UctpElements::SESSION_ID, session->getUid());   
}

//////////////////////////////////////////////////////////////////////////

void GetRemoteTaskStatusRequest::formHeader( QXmlStreamWriter& stream )
{
    stream.writeTextElement(UctpElements::SESSION_ID, session->getUid()); 

}

void GetRemoteTaskStatusRequest::formContents( QXmlStreamWriter& stream )
{
    QString id = QString("task%1").arg(taskId, 0, 16);
    stream.writeTextElement(UctpElements::TASK_ID, id);
    if (! properties.isEmpty()) {
        QString propRequest = properties.join(",");
        stream.writeTextElement(UctpElements::PROPERTY, propRequest);
    } 
}

void UpdateRemoteTaskRequest::formHeader( QXmlStreamWriter& stream )
{
    stream.writeTextElement(UctpElements::SESSION_ID, session->getUid());  
}

void UpdateRemoteTaskRequest::formContents( QXmlStreamWriter& stream )
{
    QString id = QString("task%1").arg(taskId, 0, 16);
    stream.writeTextElement(UctpElements::TASK_ID, id);
}


} // ~namespace
