/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _UCTP_REQUEST_BUILDERS_H_
#define _UCTP_REQUEST_BUILDERS_H_

#include "CloudTransportProtocol.h"

namespace GB2 {

class InitSessionRequest : public UctpRequestBuilder 
{
    QString name;
    QString pass;
public:
    InitSessionRequest(const UctpCommand& cmd, const QString& username, const QString& passwd);
    virtual void formHeader(QXmlStreamWriter& stream);
    virtual void formContents(QXmlStreamWriter& stream);
};

class GetServiceListRequest : public UctpRequestBuilder
{
public:
    GetServiceListRequest(const UctpCommand& cmd) : UctpRequestBuilder(cmd) {}
    virtual void formHeader(QXmlStreamWriter& stream) {}
    virtual void formContents(QXmlStreamWriter& writer) {}
};

class PingRequest : public UctpRequestBuilder {
public:
    PingRequest(const UctpCommand& cmd) : UctpRequestBuilder(cmd) {}
    virtual void formHeader(QXmlStreamWriter& writer) {}
    virtual void formContents(QXmlStreamWriter& writer) {}
};

class RunRemoteTaskRequest : public UctpRequestBuilder {
    UctpSession* session;
    const QByteArray& taskSchema;
    const QStringList& inputUrls;
    const QStringList& resultUrls;
    QMap<QString, QByteArray> dataMap;
public:
    RunRemoteTaskRequest(const UctpCommand& cmd, UctpSession* session, const QByteArray& task, 
           const QStringList& input, const QStringList& res);
    virtual void formHeader(QXmlStreamWriter& writer);
    virtual void formContents(QXmlStreamWriter& writer);
};

class EndSessionRequest : public UctpRequestBuilder {   
    UctpSession* session;
public:
    EndSessionRequest(const UctpCommand& cmd, UctpSession* s) : UctpRequestBuilder(cmd), session(s) {}
    virtual void formHeader(QXmlStreamWriter& writer);
    virtual void formContents(QXmlStreamWriter& writer) {}

};

class GetRemoteTaskStatusRequest : public UctpRequestBuilder {
    UctpSession* session;
    qint64 taskId;
    QStringList properties;
public:
    GetRemoteTaskStatusRequest(const UctpCommand& cmd, UctpSession* s,  qint64 id, const QStringList& props = QStringList()) 
        : UctpRequestBuilder(cmd), session(s), taskId(id), properties(props) {}
    virtual void formHeader(QXmlStreamWriter& stream);
    virtual void formContents(QXmlStreamWriter& writer); 
};


class UpdateRemoteTaskRequest : public UctpRequestBuilder {
    UctpSession* session;
    qint64 taskId;
public:
    UpdateRemoteTaskRequest(const UctpCommand& cmd, UctpSession* s, qint64 id) 
        : UctpRequestBuilder(cmd), session(s), taskId(id) {}
    virtual void formHeader(QXmlStreamWriter& stream);
    virtual void formContents(QXmlStreamWriter& stream); 
};



} // namespace


#endif // _UCTP_REQUEST_BUILDERS_H_