/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "ExternalToolRunTask.h"

#include <core_api/AppContext.h>
#include <core_api/ExternalToolRegistry.h>
#include <core_api/Log.h>
#include <QtCore/QString>

namespace GB2 {

static LogCategory log(ULOG_CAT_EXTERNAL_TOOL_RUN_TASK);

ExternalToolRunTask::ExternalToolRunTask(const QString& _toolName, const QStringList& _arguments, ExternalToolLogParser*  _logParser) :
        Task(_toolName + " run task", TaskFlag_None), arguments(_arguments), logParser(_logParser), toolName(_toolName)
{
    program=AppContext::getExternalToolRegistry()->getByName(toolName)->getPath();
    log.trace("Create "+toolName+" run task");
    logData=(char*)malloc(1000*sizeof(char));
    externalToolProcess=NULL;
}
ExternalToolRunTask::~ExternalToolRunTask(){
    free(logData);
    logData=NULL;
    delete externalToolProcess;
    externalToolProcess=NULL;
}
void ExternalToolRunTask::prepare(){
    externalToolProcess=new QProcess();//???
    connect(externalToolProcess,SIGNAL(readyReadStandardOutput()),SLOT(sl_onReadyToReadLog()));
    connect(externalToolProcess,SIGNAL(readyReadStandardError()),SLOT(sl_onReadyToReadErrLog()));
}
void ExternalToolRunTask::run(){
    externalToolProcess->start(program, arguments);
    //externalToolProcess->state()
    if(!externalToolProcess->waitForStarted(3000)){
        stateInfo.setError(tr("Can not run %1 tool.").arg(toolName));
        return;
    }
    while(!externalToolProcess->waitForFinished(1000)){
        if (isCanceled()) {
            cancelProcess();
        }
    }
}
Task::ReportResult ExternalToolRunTask::report(){
    return ReportResult_Finished;
}
void ExternalToolRunTask::cancelProcess(){
    externalToolProcess->kill();
}

void ExternalToolRunTask::sl_onReadyToReadLog(){
    if(externalToolProcess->readChannel() == QProcess::StandardError)
        externalToolProcess->setReadChannel(QProcess::StandardOutput);
    int numberReadChars=externalToolProcess->read(logData,1000);
    while(numberReadChars > 0){
        //call log parser
        logParser->parseOutput(QString(logData).left(numberReadChars));
        numberReadChars=externalToolProcess->read(logData,1000);
    }
    stateInfo.progress=logParser->getProgress();
    emit si_progressChanged();
}

void ExternalToolRunTask::sl_onReadyToReadErrLog(){
    if(externalToolProcess->readChannel() == QProcess::StandardOutput)
        externalToolProcess->setReadChannel(QProcess::StandardError);
    int numberReadChars=externalToolProcess->read(logData,1000);
    while(numberReadChars > 0){
        //call log parser
        logParser->parseErrOutput(QString(logData).left(numberReadChars));
        numberReadChars=externalToolProcess->read(logData,1000);
    }
    stateInfo.progress=logParser->getProgress();
    emit si_progressChanged();
}
////////////////////////////////////////
//ExternalToolLogParser
ExternalToolLogParser::ExternalToolLogParser() {
    progress=-1;
    lastLine="";
    lastErrLine="";
}
void ExternalToolLogParser::parseOutput(const QString& partOfLog){
    lastPartOfLog=partOfLog.split(QChar('\n'));
    lastPartOfLog.first()=lastLine+lastPartOfLog.first();
    lastLine=lastPartOfLog.takeLast();
    foreach(QString buf, lastPartOfLog){
        log.info(buf);
    }
}

void ExternalToolLogParser::parseErrOutput(const QString& partOfLog){
    lastPartOfLog=partOfLog.split(QChar('\n'));
    lastPartOfLog.first()=lastErrLine+lastPartOfLog.first();
    lastErrLine=lastPartOfLog.takeLast();
    foreach(const QString& buf, lastPartOfLog){
        log.error(buf);
    }
}
}//namespace
