/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include <core_api/AppContext.h>

#include <core_api/DNAAlphabet.h>
#include <core_api/DataBaseRegistry.h>
#include <core_api/ExternalToolRegistry.h>

#include <util_ov_msaedit/MSAEditor.h>
#include <util_ov_msaedit/MSAEditorFactory.h>
#include <gobjects/MAlignmentObject.h>

#include <util_algorithm/GAutoDeleteList.h>
#include <util_ov_annotated_dna/AnnotatedDNAView.h>
#include <util_ov_annotated_dna/ADVSequenceObjectContext.h>
#include <util_ov_annotated_dna/ADVConstants.h>
#include <util_ov_annotated_dna/ADVUtils.h>

#include <selection/DNASequenceSelection.h>
#include <gobjects/DNASequenceObject.h>
#include <util_gui/GUIUtils.h>

#include <test_framework/xmltest/XMLTestFormat.h>
#include <test_framework/GTest.h>
#include <test_framework/GTestFrameworkComponents.h>

#include <QtGui/QMenu>
#include <QtCore/QCoreApplication>
#include <QtCore/QDir>
#include <QtCore/QDirIterator>
#include <QtCore/QFileInfo>
#include <QtGui/QMessageBox>

#include "ExternalToolSupportPlugin.h"
#include "ExternalToolSupportSettings.h"
#include "utils/ExternalToolSupportAction.h"

#include "clustalw/ClustalWSupport.h"
#include "clustalw/ClustalWWorker.h"
#include "mafft/MAFFTSupport.h"
#include "mafft/MAFFTWorker.h"

#define EXTERNAL_TOOL_SUPPORT_FACTORY_ID "ExternalToolSupport"

namespace GB2 {

extern "C" Q_DECL_EXPORT Plugin * GB2_PLUGIN_INIT_FUNC() {
    if (AppContext::getMainWindow()) {
        ExternalToolSupportPlugin * plug = new ExternalToolSupportPlugin();
        return plug;
    }
    return NULL;
}

ExternalToolSupportPlugin::ExternalToolSupportPlugin():Plugin(tr("External tool support"),tr("Runs other external tools")) {

    //Fill ExternalToolRegistry with supported tools
    //ClustalW
    ClustalWSupport* clustalWTool=new ClustalWSupport(CLUSTAL_TOOL_NAME);
    AppContext::getExternalToolRegistry()->registerEntry(clustalWTool);

    clustalWTool->getViewContext()->setParent(this);
    clustalWTool->getViewContext()->init();

    ExternalToolSupprotAction* clustalWAction = new ExternalToolSupprotAction("ClustalW", this, CLUSTAL_TOOL_NAME);
    connect(clustalWAction, SIGNAL(triggered()), clustalWTool, SLOT(sl_runWithExtFileSpecify()));

    //MAFFT
    MAFFTSupport* mAFFTTool=new MAFFTSupport(MAFFT_TOOL_NAME);
    AppContext::getExternalToolRegistry()->registerEntry(mAFFTTool);

    mAFFTTool->getViewContext()->setParent(this);
    mAFFTTool->getViewContext()->init();

    ExternalToolSupprotAction* mAFFTAction= new ExternalToolSupprotAction("MAFFT", this, MAFFT_TOOL_NAME);
    connect(mAFFTAction, SIGNAL(triggered()), mAFFTTool, SLOT(sl_runWithExtFileSpecify()));

    //Add to menu MA
    MWMenuManager* menu = AppContext::getMainWindow()->getMenuManager();
    QMenu* tools = menu->getTopLevelMenu(MWMENU_TOOLS);
    QMenu* toolsSubmenu = tools->findChild<QMenu*>(MWMENU_TOOLS_MALIGN);
    if(toolsSubmenu == NULL){
        toolsSubmenu = tools->addMenu(tr("Multiple Alignment"));
        toolsSubmenu->setObjectName(MWMENU_TOOLS_MALIGN);
    }
    toolsSubmenu->setIcon(QIcon(":external_tool_support/images/ma.png"));//bad code
    toolsSubmenu->addAction(clustalWAction);
    toolsSubmenu->addAction(mAFFTAction);

    //Read settings
    ExternalToolSupportSettings::getExternalTools();
    AppContext::getExternalToolRegistry()->setTemporaryDir(ExternalToolSupportSettings::getTemporyDirectory());
    //Search in tools path
#ifdef Q_OS_WIN
    QString clustalWPath = QCoreApplication::applicationDirPath()+"\\tools\\clustalw2.exe";
    if(AppContext::getExternalToolRegistry()->getByName(CLUSTAL_TOOL_NAME)->getPath().isEmpty()){
        QFileInfo clustalExe(clustalWPath);
        if(clustalExe.exists()){
            AppContext::getExternalToolRegistry()->getByName(CLUSTAL_TOOL_NAME)->setPath(clustalWPath);
        }
        }

#endif
    //Add viewer for settings
    AppContext::getAppSettingsGUI()->registerPage(new ExternalToolSupportSettingsPageController());
    //Add new workers to WD
    LocalWorkflow::ClustalWWorkerFactory::init();
    LocalWorkflow::MAFFTWorkerFactory::init();
}

ExternalToolSupportPlugin::~ExternalToolSupportPlugin(){
    ExternalToolSupportSettings::setExternalTools();
    ExternalToolSupportSettings::setTemporyDirectory(AppContext::getExternalToolRegistry()->getTemporaryDir());
}

}
