/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "ExternalToolSupportSettings.h"
//#include "WorkflowViewController.h"

#include <core_api/AppContext.h>
#include <core_api/Settings.h>
#include <core_api/ExternalToolRegistry.h>

#include <QtCore/QSettings>
#include <QtGui/QApplication>
#include <QtGui/QStyle>
#include <QtGui/QStyleFactory>

namespace GB2 {

#define NUMBER_EXTERNAL_TOOL SETTINGS + "numberExternalTools"
#define PREFIX_EXTERNAL_TOOL_NAME SETTINGS + "exToolName"
#define PREFIX_EXTERNAL_TOOL_PATH SETTINGS + "exToolPath"
#define TEMPORY_DIRECTORY SETTINGS + "temporyDirectory"

Watcher* const ExternalToolSupportSettings::watcher = new Watcher;

int ExternalToolSupportSettings::prevNumberExternalTools = 0;

int ExternalToolSupportSettings::getNumberExternalTools() {
    return AppContext::getSettings()->getValue(NUMBER_EXTERNAL_TOOL, 0).toInt();
}

void ExternalToolSupportSettings::setNumberExternalTools( int v ) {
    AppContext::getSettings()->setValue(NUMBER_EXTERNAL_TOOL, v);
    emit watcher->changed();
}

bool ExternalToolSupportSettings::getExternalTools() {
    int numberExternalTools=getNumberExternalTools();
    QString name;
    QString path;
    for(int i=0; i<numberExternalTools;i++){
        name=AppContext::getSettings()->getValue(PREFIX_EXTERNAL_TOOL_NAME + QString::number(i), 0).toString();
        path=AppContext::getSettings()->getValue(PREFIX_EXTERNAL_TOOL_PATH + QString::number(i), 0).toString();
        AppContext::getExternalToolRegistry()->getByName(name)->setPath(path);
    }
    prevNumberExternalTools = numberExternalTools;
    return true;//bad code
}

void ExternalToolSupportSettings::setExternalTools() {
    QList<ExternalTool*> ExternalToolList = AppContext::getExternalToolRegistry()->getAllEntries();
    int numberExternalTools = ExternalToolList.length();
    setNumberExternalTools(numberExternalTools);
    QString name;
    QString path;
    int numberIterations=numberExternalTools >= prevNumberExternalTools ? numberExternalTools : prevNumberExternalTools;
    for(int i=0; i<numberIterations;i++){
        if(i<numberExternalTools){
            name = ExternalToolList.at(i)->getName();
            path = ExternalToolList.at(i)->getPath();
            AppContext::getSettings()->setValue(PREFIX_EXTERNAL_TOOL_NAME + QString::number(i), name);
            AppContext::getSettings()->setValue(PREFIX_EXTERNAL_TOOL_PATH + QString::number(i), path);
        }else{
            AppContext::getSettings()->remove(PREFIX_EXTERNAL_TOOL_NAME + QString::number(i));
            AppContext::getSettings()->remove(PREFIX_EXTERNAL_TOOL_PATH + QString::number(i));
        }
    }
    prevNumberExternalTools = numberExternalTools;
}

QString ExternalToolSupportSettings::getTemporyDirectory() {
    return AppContext::getSettings()->getValue(TEMPORY_DIRECTORY, "").toString();
}

void ExternalToolSupportSettings::setTemporyDirectory(const QString& v) {
    AppContext::getSettings()->setValue(TEMPORY_DIRECTORY, v);
    emit watcher->changed();
}

}//namespace
