/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "ExternalToolSupportSettingsController.h"
#include "ExternalToolSupportSettings.h"

#include <core_api/AppContext.h>
#include <util_gui/DialogUtils.h>
#include <QtGui>

//#include <core_api/Log.h>

namespace GB2
{
//static LogCategory log(ULOG_CAT_USER_INTERFACE);

ExternalToolSupportSettingsPageController::ExternalToolSupportSettingsPageController(QObject* p)
: AppSettingsGUIPageController(tr("External tool support"), ExternalToolSupportSettingsPageId, p) {}


AppSettingsGUIPageState* ExternalToolSupportSettingsPageController::getSavedState() {
    ExternalToolSupportSettingsPageState* state = new ExternalToolSupportSettingsPageState();
    state->externalTools=AppContext::getExternalToolRegistry()->getAllEntries();
    state->temporaryDirectory=AppContext::getExternalToolRegistry()->getTemporaryDir();
    return state;
}

void ExternalToolSupportSettingsPageController::saveState(AppSettingsGUIPageState* s) {

    ExternalToolSupportSettingsPageState* state = qobject_cast<ExternalToolSupportSettingsPageState*>(s);
    //Add new external tools to registry
    foreach(ExternalTool* externalTool, state->externalTools){
        AppContext::getExternalToolRegistry()->getByName(externalTool->getName())->setPath(externalTool->getPath());
    }
    AppContext::getExternalToolRegistry()->setTemporaryDir(state->temporaryDirectory);
    ExternalToolSupportSettings::setTemporyDirectory(state->temporaryDirectory);
    ExternalToolSupportSettings::setExternalTools();
}

AppSettingsGUIPageWidget* ExternalToolSupportSettingsPageController::createWidget(AppSettingsGUIPageState* state) {
    ExternalToolSupportSettingsPageWidget* r = new ExternalToolSupportSettingsPageWidget(this);
    r->setState(state);
    return r;
}


ExternalToolSupportSettingsPageWidget::ExternalToolSupportSettingsPageWidget(ExternalToolSupportSettingsPageController* ) {
    setupUi(this);
    connect(temporaryDirButton,SIGNAL(clicked()),SLOT(sl_onBrowseTemporyDir()));
}

void ExternalToolSupportSettingsPageWidget::sl_onBrowseTemporyDir() {
    LastOpenDirHelper lod("TmpDir");

    QString name;
    lod.url = name = QFileDialog::getExistingDirectory(NULL, tr("Select a directory for tempory files"), lod.dir);
    if (!name.isEmpty()) {
        temporaryDirLineEdit->setText(name);
    }
    setFocus();
}

QWidget* ExternalToolSupportSettingsPageWidget::createEditor(QWidget *parent, const QString& path) const
{
    QWidget * widget = new QWidget(parent);
    PathLineEdit* toolPathEdit= new PathLineEdit("","executable", false, widget);
    toolPathEdit->setObjectName("PathLineEdit");
    toolPathEdit->setFrame(false);
    toolPathEdit->setSizePolicy(QSizePolicy(QSizePolicy::Preferred,QSizePolicy::Preferred));
    toolPathEdit->setReadOnly(true);
    toolPathEdit->setText(path);
    widget->setFocusProxy(toolPathEdit);
    QToolButton * selectToolPathButton = new QToolButton(widget);
    selectToolPathButton->setVisible(true);
    selectToolPathButton->setText("...");
    selectToolPathButton->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred));
    connect(selectToolPathButton, SIGNAL(clicked()), toolPathEdit, SLOT(sl_onBrowse()));

    QToolButton * clearToolPathButton = new QToolButton(widget);
    clearToolPathButton->setObjectName("ClearToolPathButton");
    clearToolPathButton->setVisible(true);
    clearToolPathButton->setIcon(QIcon(":external_tool_support/images/editdelete.png"));
    clearToolPathButton->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred));
    clearToolPathButton->setEnabled(!path.isEmpty());
    connect(clearToolPathButton,SIGNAL(clicked()),toolPathEdit,SLOT(sl_clear()));

    QHBoxLayout* layout = new QHBoxLayout(widget);
    layout->setSpacing(0);
    layout->setMargin(0);
    layout->addWidget(toolPathEdit);
    layout->addWidget(selectToolPathButton);
    layout->addWidget(clearToolPathButton);

    return widget;
}

void ExternalToolSupportSettingsPageWidget::setState(AppSettingsGUIPageState* s) {
    ExternalToolSupportSettingsPageState* state = qobject_cast<ExternalToolSupportSettingsPageState*>(s);

    tableWidget->setRowCount(state->externalTools.length());
    tableWidget->setColumnWidth(0,this->geometry().width()/3);
    tableWidget->horizontalHeader()->setStretchLastSection(true);
    tableWidget->horizontalHeader()->setClickable(false);

    for (int i=0; i < state->externalTools.length();i++){
        tableWidget->setItem( i,0, new QTableWidgetItem(state->externalTools.at(i)->getName()));
        tableWidget->setCellWidget(i,1,createEditor(tableWidget, state->externalTools.at(i)->getPath()));
    }

    temporaryDirLineEdit->setText(state->temporaryDirectory);
}

AppSettingsGUIPageState* ExternalToolSupportSettingsPageWidget::getState(QString& ) const {
    ExternalToolSupportSettingsPageState* state = new ExternalToolSupportSettingsPageState();
    ExternalTool* externalTool;
    for(int i=0; i<tableWidget->rowCount();i++){
        QLineEdit* ed = tableWidget->cellWidget(i,1)->findChild<QLineEdit*>("PathLineEdit");
        assert(ed);
        externalTool=new ExternalTool(tableWidget->item(i,0)->text(),ed->text());
        state->externalTools.append(externalTool);
    }
    state->temporaryDirectory=temporaryDirLineEdit->text();
    return state;
}

////////////////////////////////////////
//PathLineEdit
void PathLineEdit::sl_onBrowse() {
    LastOpenDirHelper lod(type);

    QString name;
    lod.url = name = QFileDialog::getOpenFileName(NULL, tr("Select a file"), lod.dir, FileFilter, 0, QFileDialog::DontConfirmOverwrite);
    if (!name.isEmpty()) {
        setText(name);
    }
    QToolButton* clearToolPathButton = this->parentWidget()->findChild<QToolButton*>("ClearToolPathButton");
    assert(clearToolPathButton);
    clearToolPathButton->setEnabled(!text().isEmpty());
    setFocus();
}

void PathLineEdit::sl_clear(){
    QToolButton* s=qobject_cast<QToolButton*>(sender());
    assert(s);
    setText("");
    s->setEnabled(false);
}

} //namespace
