/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "SelectPathDialog.h"
#include "ui_SelectPathDialog.h"

#include <core_api/AppContext.h>
#include <core_api/MainWindow.h>
#include <core_api/ExternalToolRegistry.h>
#include <util_gui/DialogUtils.h>

#include <QtGui/QMainWindow>
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>

namespace GB2
{

SelectPathDialog::SelectPathDialog(const QString& _externalToolName, QWidget *parent) :
    QDialog(parent), externalToolName(_externalToolName)
{
    setupUi(this);
    externalToolPathLabel->setText(externalToolPathLabel->text()+externalToolName+":");
    externalToolPathLineEdit->setText(AppContext::getExternalToolRegistry()->getByName(externalToolName)->getPath());
    temporaryDirLineEdit->setText(AppContext::getExternalToolRegistry()->getTemporaryDir());

    connect(externalToolPathButton,SIGNAL(clicked()),SLOT(sl_onBrowseExternalTool()));
    connect(temporaryDirButton,SIGNAL(clicked()),SLOT(sl_onBrowseTemporaryDir()));
    connect(externalToolPathLineEdit,SIGNAL(textChanged(QString)),this,SLOT(sl_onPathsChanged()));
    connect(temporaryDirLineEdit,SIGNAL(textChanged(QString)),this,SLOT(sl_onPathsChanged()));
    connect(okButton,SIGNAL(clicked()),SLOT(sl_onOkButtonClick()));
    connect(cancelButton,SIGNAL(clicked()),SLOT(reject()));

    okButton->setEnabled((!temporaryDirLineEdit->text().isEmpty())&&(!externalToolPathLineEdit->text().isEmpty()));
}

SelectPathDialog::~SelectPathDialog()
{}

void SelectPathDialog::sl_onBrowseExternalTool() {
    LastOpenDirHelper lod("executable");

    QString name;
    lod.url = name = QFileDialog::getOpenFileName(NULL, tr("Select a file"), lod.dir, "", 0, QFileDialog::DontConfirmOverwrite);
    if (!name.isEmpty()) {
        externalToolPathLineEdit->setText(name);
    }
    okButton->setEnabled((!name.isEmpty())&&(!temporaryDirLineEdit->text().isEmpty()));
    setFocus();
}

void SelectPathDialog::sl_onBrowseTemporaryDir() {
    LastOpenDirHelper lod("TmpDir");

    QString name;
    lod.url = name = QFileDialog::getExistingDirectory(NULL, tr("Select a directory for tempory files"), lod.dir);
    if (!name.isEmpty()) {
        temporaryDirLineEdit->setText(name);
    }
    okButton->setEnabled((!name.isEmpty())&&(!externalToolPathLineEdit->text().isEmpty()));
    setFocus();
}
void SelectPathDialog::sl_onPathsChanged() {
    okButton->setEnabled(!(externalToolPathLineEdit->text().isEmpty()||temporaryDirLineEdit->text().isEmpty()));
}

void SelectPathDialog::sl_onOkButtonClick() {
    assert(!externalToolPathLineEdit->text().isEmpty());
    assert(!temporaryDirLineEdit->text().isEmpty());
    QDir tmpDir(temporaryDirLineEdit->text());
    if(tmpDir.exists()){
        AppContext::getExternalToolRegistry()->setTemporaryDir(temporaryDirLineEdit->text());
    }else{
        QMessageBox createTmpDirMsgBox;
        createTmpDirMsgBox.setText(tr("The directory %1 does not exist.").arg(temporaryDirLineEdit->text()));
        createTmpDirMsgBox.setInformativeText(tr("Do you want to create this directory?"));
        createTmpDirMsgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        createTmpDirMsgBox.setDefaultButton(QMessageBox::Yes);
        if(createTmpDirMsgBox.exec()==QMessageBox::Yes){
            if(!tmpDir.mkpath(temporaryDirLineEdit->text())){
                QMessageBox notCreatedTmpDirMsgBox;
                notCreatedTmpDirMsgBox.setText(tr("Can not create directory for temporary files."));
                notCreatedTmpDirMsgBox.setInformativeText(tr("Try choose another directory."));
                notCreatedTmpDirMsgBox.exec();
                return;
            }
        }else{
            return;
        }
        //QMessa
    }
    AppContext::getExternalToolRegistry()->getByName(externalToolName)->setPath(externalToolPathLineEdit->text());
    accept();
}

}//namespace
