/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "MAFFTSupport.h"
#include "MAFFTSupportRunDialog.h"
#include "MAFFTSupportTask.h"
#include "SelectPathDialog.h"

#include <core_api/AppContext.h>
#include <core_api/MainWindow.h>
#include <QtGui/QMainWindow>
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>
#include <util_ov_msaedit/MSAEditor.h>
#include <util_ov_msaedit/MSAEditorFactory.h>

#include <gobjects/MAlignmentObject.h>

#include <util_gui/GUIUtils.h>
#include <util_gui/DialogUtils.h>

namespace GB2 {


MAFFTSupport::MAFFTSupport(const QString& name, const QString& path) : ExternalTool(name, path)
{
    viewCtx = new MAFFTSupportContext(this);
    icon=QIcon(":external_tool_support/images/cmdline.png");
    grayIcon=QIcon(":external_tool_support/images/cmdline_gray.png");
}

void MAFFTSupport::sl_runWithExtFileSpecify(){
    //Check that Clustal and tempory directory path defined
    if ((AppContext::getExternalToolRegistry()->getByName(MAFFT_TOOL_NAME)->getPath().isEmpty()) ||
        (AppContext::getExternalToolRegistry()->getTemporaryDir().isEmpty()))
    {
        SelectPathDialog selectPathDialog(MAFFT_TOOL_NAME, AppContext::getMainWindow()->getQMainWindow());
        if(selectPathDialog.exec() != QDialog::Accepted){
            return;
        }
    }
    //Call select input file and setup settings dialog
    MAFFTSupportTaskSettings settings;
    MAFFTWithExtFileSpecifySupportRunDialog mAFFTRunDialog(settings, AppContext::getMainWindow()->getQMainWindow());
    if(mAFFTRunDialog.exec() != QDialog::Accepted){
        return;
    }
    assert(!settings.inputFilePath.isEmpty());
    //
    MAFFTWithExtFileSpecifySupportTask* mAFFTSupportTask=new MAFFTWithExtFileSpecifySupportTask(settings);
    AppContext::getTaskScheduler()->registerTopLevelTask(mAFFTSupportTask);
}

////////////////////////////////////////
//ClustalWSupportAction ???    
MSAEditor* MAFFTSupportAction::getMSAEditor() const {
        MSAEditor* e = qobject_cast<MSAEditor*>(getObjectView());
        assert(e!=NULL);
        return e;
}

void MAFFTSupportAction::sl_lockedStateChanged() {
        StateLockableItem* item = qobject_cast<StateLockableItem*>(sender());
        assert(item!=NULL);
        setEnabled(!item->isStateLocked());
}
////////////////////////////////////////
//ExternalToolSupportMSAContext
MAFFTSupportContext::MAFFTSupportContext(QObject* p) : GObjectViewWindowContext(p, MSAEditorFactory::ID) {

}

void MAFFTSupportContext::initViewContext(GObjectView* view) {
    MSAEditor* msaed = qobject_cast<MSAEditor*>(view);
    assert(msaed!=NULL);
    if (msaed->getMSAObject() == NULL) {
            return;
    }
    bool objLocked = msaed->getMSAObject()->isStateLocked();

    MAFFTSupportAction* alignAction = new MAFFTSupportAction(this, view, tr("Align with MAFFT"), 2000);

    addViewAction(alignAction);
    alignAction->setEnabled(!objLocked);

    connect(msaed->getMSAObject(), SIGNAL(si_lockedStateChanged()), alignAction, SLOT(sl_lockedStateChanged()));
    connect(alignAction, SIGNAL(triggered()), SLOT(sl_align_with_MAFFT()));
}
void MAFFTSupportContext::buildMenu(GObjectView* view, QMenu* m) {
        QList<GObjectViewAction *> actions = getViewActions(view);
        QMenu* alignMenu = GUIUtils::findSubMenu(m, MSAE_MENU_ALIGN);
        assert(alignMenu!=NULL);
        foreach(GObjectViewAction* a, actions) {
                a->addToMenuWithOrder(alignMenu);
        }
}
void MAFFTSupportContext::sl_align_with_MAFFT() {
    //Check that MAFFT and tempory directory path defined
    if ((AppContext::getExternalToolRegistry()->getByName(MAFFT_TOOL_NAME)->getPath().isEmpty()) ||
        (AppContext::getExternalToolRegistry()->getTemporaryDir().isEmpty()))
    {
        SelectPathDialog selectPathDialog(MAFFT_TOOL_NAME, AppContext::getMainWindow()->getQMainWindow());
        if(selectPathDialog.exec() != QDialog::Accepted){
            return;
        }
    }
    //Call run MAFFT align dialog
    MAFFTSupportAction* action = qobject_cast<MAFFTSupportAction*>(sender());
    assert(action!=NULL);
    MSAEditor* ed = action->getMSAEditor();
    MAlignmentObject* obj = ed->getMSAObject();
    if (obj == NULL)
            return;
    assert(!obj->isStateLocked());

    MAFFTSupportTaskSettings settings;
    MAFFTSupportRunDialog mAFFTRunDialog(settings, AppContext::getMainWindow()->getQMainWindow());
    if(mAFFTRunDialog.exec() != QDialog::Accepted){
        return;
    }

    MAFFTSupportTask* mAFFTSupportTask=new MAFFTSupportTask(obj, settings);
    AppContext::getTaskScheduler()->registerTopLevelTask(mAFFTSupportTask);

}

}//namespace
