/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "RFTaskFactory.h"
#include "RFBase.h"
#include "FindRepeatsTask.h"

namespace GB2 {

Task* RFTaskFactory::getTaskInstance( const RepeatFinderSettings& c) const {

	if (c._inverted) {
		return new ReverseAndCreateTask(c);
/*
		result = new Task("Make reverse sequence and find repeats", TaskFlag_NoRun);

		RevComplSequenceTask *revTask = new RevComplSequenceTask(DNASequence(QByteArray(c._seqX), c._al), LRegion(0, c._sizeX));
		result->addSubTask(revTask);

		Task *rfBase = RFAlgorithmBase::createTask(
			c._l,
			revTask->complementSequence.constData(),
			c._sizeX,
			c._seqY,
			c._sizeY,
			c._al,
			c._w,
			c._mismatches,
			c._alg,
			c._nThreads
		);
		result->addSubTask(rfBase);
		*/
	}
	else {
		return RFAlgorithmBase::createTask(
			c._l,
			c._seqX,
			c._sizeX,
			c._seqY,
			c._sizeY,
			c._al,
			c._w,
			c._mismatches,
			c._alg,
			c._nThreads
		);
	}
}

void RFTaskFactory::setRFResultsListener(Task* b, RFResultsListener* l) {
	RFAlgorithmBase* rfTask = qobject_cast<RFAlgorithmBase*>(b);
	if (rfTask) {
		rfTask->setRFResultsListener(l);
	}
}

ReverseAndCreateTask::ReverseAndCreateTask(const RepeatFinderSettings& c)
: Task("Make reverse sequence and find repeats", TaskFlag_NoRun)
{
	this->c = c;

	revTask = new RevComplSequenceTask(DNASequence(QByteArray(c._seqX), c._al), LRegion(0, c._sizeX));
	addSubTask(revTask);
}

QList<Task*> ReverseAndCreateTask::onSubTaskFinished(Task* subTask) {

	QList<Task*> subTasks;

	if (revTask == subTask) {

		Task *rfBase = RFAlgorithmBase::createTask(
			c._l,
			revTask->complementSequence.constData(),
			c._sizeX,
			c._seqY,
			c._sizeY,
			c._al,
			c._w,
			c._mismatches,
			c._alg,
			c._nThreads
		);
		subTasks.append(rfBase);
	}

	return subTasks;
}

} // namespace
